@@ -1,7 +1,7 @@
 #
 # Run this Tcl script to generate the pragma.html file.
 #
-set rcsid {$Id: pragma.tcl,v 1.12 2005/03/21 01:31:37 drh Exp $}
+set rcsid {$Id: pragma.tcl,v 1.16 2005/08/28 17:00:26 drh Exp $}
 source common.tcl
 header {Pragma statements supported by SQLite}
 
@@ -108,6 +108,16 @@ puts {
     the <a href="#pragma_default_cache_size"><b>default_cache_size</b></a> 
     pragma to check the cache size permanently.</p></li>
 
+<a name="pragma_case_sensitive_like"></a>
+<li><p><b>PRAGMA case_sensitive_like;
+       <br>PRAGMA case_sensitive_like = </b><i>0 | 1</i><b>;</b></p>
+    <p>The default behavior of the LIKE operator is to ignore case
+    for latin1 characters. Hence, by default <b>'a' LIKE 'A'</b> is
+    true.  The case_sensitive_like pragma can be turned on to change
+    this behavior.  When case_sensitive_like is enabled,
+    <b>'a' LIKE 'A'</b> is false but <b>'a' LIKE 'a'</b> is still true.</p>
+    </li>
+
 <a name="pragma_count_changes"></a>
 <li><p><b>PRAGMA count_changes;
        <br>PRAGMA count_changes = </b><i>0 | 1</i><b>;</b></p>
@@ -160,7 +170,7 @@ puts {
        <br>PRAGMA encoding = "UTF-16";
        <br>PRAGMA encoding = "UTF-16le";
        <br>PRAGMA encoding = "UTF-16be";</b></p>
-    <p>In it's first form, if the main database has already been
+    <p>In first form, if the main database has already been
     created, then this pragma returns the text encoding used by the
     main database, one of "UTF-8", "UTF-16le" (little-endian UTF-16
     encoding) or "UTF-16be" (big-endian UTF-16 encoding).  If the main
@@ -171,7 +181,12 @@ puts {
     the main database has not already been created. In this case the 
     pragma sets the encoding that the main database will be created with if
     it is created by this session. The string "UTF-16" is interpreted
-    as "UTF-16 encoding using native machine byte-ordering".</p>
+    as "UTF-16 encoding using native machine byte-ordering".  If the second
+    and subsequent forms are used after the database file has already
+    been created, they have no effect and are silently ignored.</p>
+
+    <p>Once an encoding has been set for a database, it cannot be changed.</p>
+
     <p>Databases created by the ATTACH command always use the same encoding
     as the main database.</p>
 </li>
@@ -203,7 +218,8 @@ puts {
     may only be set if the database has not yet been created. The page
     size must be a power of two greater than or equal to 512 and less
     than or equal to 8192. The upper limit may be modified by setting
-    the value of macro SQLITE_MAX_PAGE_SIZE during compilation.
+    the value of macro SQLITE_MAX_PAGE_SIZE during compilation.  The
+    maximum upper bound is 32768.
     </p>
 </li>
 
@@ -239,13 +255,13 @@ puts {
     the operating system crashes or if there is a power failure, the database
     will be uncorrupted after rebooting.  FULL synchronous is very 
     safe, but it is also slow.  
-    When synchronous is NORMAL (1, the default), the SQLite database
+    When synchronous is NORMAL, the SQLite database
     engine will still pause at the most critical moments, but less often
     than in FULL mode.  There is a very small (though non-zero) chance that
     a power failure at just the wrong time could corrupt the database in
     NORMAL mode.  But in practice, you are more likely to suffer
     a catastrophic disk failure or some other unrecoverable hardware
-    fault.  So NORMAL is the default mode.
+    fault.
     With synchronous OFF (0), SQLite continues without pausing
     as soon as it has handed data off to the operating system.
     If the application running SQLite crashes, the data will be safe, but
@@ -253,7 +269,11 @@ puts {
     crashes or the computer loses power before that data has been written
     to the disk surface.  On the other hand, some
     operations are as much as 50 or more times faster with synchronous OFF.
-    </p></li>
+    </p>
+    <p>In SQLite version 2, the default value is NORMAL. For version 3, the
+    default was changed to FULL.
+    </p>
+</li>
 
 
 <a name="pragma_temp_store"></a>
